@@ -1,9 +1,11 @@
 use std::sync::Arc;
+use std::sync::Mutex;
 use std::time::SystemTime;
 
 use structopt::StructOpt;
 
 use syndicate::actor::*;
+use syndicate::enclose;
 use syndicate::language;
 use syndicate::relay;
 use syndicate::schemas::dataspace::Observe;
@@ -14,7 +16,6 @@ use syndicate::value::Value;
 use tokio::net::TcpStream;
 
 use core::time::Duration;
-use tokio::time::interval;
 
 #[derive(Clone, Debug, StructOpt)]
 pub struct PingConfig {
@@ -93,7 +94,7 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let config = Config::from_args();
     let sturdyref = sturdy::SturdyRef::from_hex(&config.dataspace)?;
     let (i, o) = TcpStream::connect("127.0.0.1:8001").await?.into_split();
-    Actor::new(None).boot(syndicate::name!("pingpong"), |t| {
+    Actor::top(None, |t| {
         relay::connect_stream(t, i, o, false, sturdyref, (), move |_state, t, ds| {
 
             let (send_label, recv_label, report_latency_every, should_echo, bytes_padding) =
@@ -110,22 +111,18 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
                 let mut event_counter: u64 = 0;
                 let mut rtt_ns_samples: Vec<u64> = vec![0; report_latency_every];
                 let mut rtt_batch_count: usize = 0;
-                let mut current_reply = None;
-                let self_ref = t.create_inert();
-                self_ref.become_entity(
-                    syndicate::entity(Arc::clone(&self_ref))
-                        .on_message(move |self_ref, t, m: AnyValue| {
+                let current_reply = Arc::new(Mutex::new(None));
+                Cap::new(&t.create(
+                    syndicate::entity(())
+                        .on_message(move |(), t, m: AnyValue| {
                             match m.value().as_boolean() {
-                                Some(true) => {
+                                Some(_) => {
                                     tracing::info!("{:?} turns, {:?} events in the last second",
                                                    turn_counter,
                                                    event_counter);
                                     turn_counter = 0;
                                     event_counter = 0;
                                 }
-                                Some(false) => {
-                                    current_reply = None;
-                                }
                                 None => {
                                     event_counter += 1;
                                     let bindings = m.value().to_sequence()?;
@@ -137,9 +134,13 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
                                                                            timestamp.clone(),
                                                                            padding.clone()));
                                     } else {
-                                        if let None = current_reply {
+                                        let mut g = current_reply.lock().expect("unpoisoned");
+                                        if let None = *g {
                                             turn_counter += 1;
-                                            t.message_for_myself(&self_ref, AnyValue::new(false));
+                                            t.pre_commit(enclose!((current_reply) move |_| {
+                                                *current_reply.lock().expect("unpoisoned") = None;
+                                                Ok(())
+                                            }));
                                             let rtt_ns = now() - timestamp.value().to_u64()?;
                                             rtt_ns_samples[rtt_batch_count] = rtt_ns;
                                             rtt_batch_count += 1;
@@ -150,18 +151,16 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
                                                 rtt_batch_count = 0;
                                             }
 
-                                            current_reply = Some(
-                                                simple_record2(&send_label,
-                                                               Value::from(now()).wrap(),
-                                                               padding.clone()));
+                                            *g = Some(simple_record2(&send_label,
+                                                                     Value::from(now()).wrap(),
+                                                                     padding.clone()));
                                         }
-                                        ds.message(t, &(), current_reply.as_ref().expect("some reply"));
+                                        ds.message(t, &(), g.as_ref().expect("some reply"));
                                     }
                                 }
                             }
                             Ok(())
-                        }));
-                Cap::new(&self_ref)
+                        })))
             };
 
             ds.assert(t, language(), &Observe {
@@ -172,38 +171,28 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
                 observer: Arc::clone(&consumer),
             });
 
-            t.linked_task(syndicate::name!("tick"), async move {
-                let mut stats_timer = interval(Duration::from_secs(1));
-                loop {
-                    stats_timer.tick().await;
-                    let consumer = Arc::clone(&consumer);
-                    external_event(&Arc::clone(&consumer.underlying.mailbox),
-                                   &Account::new(syndicate::name!("account")),
-                                   Box::new(move |t| t.with_entity(
-                                       &consumer.underlying,
-                                       |t, e| e.message(t, AnyValue::new(true)))))?;
-                }
-            });
+            t.every(Duration::from_secs(1), move |t| {
+                consumer.message(t, &(), &AnyValue::new(true));
+                Ok(())
+            })?;
 
             if let PingPongMode::Ping(c) = &config.mode {
+                let facet = t.facet.clone();
                 let turn_count = c.turn_count;
                 let action_count = c.action_count;
                 let account = Arc::clone(t.account());
-                t.linked_task(syndicate::name!("boot-ping"), async move {
+                t.linked_task(Some(AnyValue::symbol("boot-ping")), async move {
                     let padding = AnyValue::bytestring(vec![0; bytes_padding]);
                     for _ in 0..turn_count {
-                        let mut events: PendingEventQueue = vec![];
                         let current_rec = simple_record2(send_label,
                                                          Value::from(now()).wrap(),
                                                          padding.clone());
-                        for _ in 0..action_count {
-                            let ds = Arc::clone(&ds);
-                            let current_rec = current_rec.clone();
-                            events.push(Box::new(move |t| t.with_entity(
-                                &ds.underlying,
-                                |t, e| e.message(t, current_rec))));
-                        }
-                        external_events(&ds.underlying.mailbox, &account, events)?
+                        facet.activate(&account, None, |t| {
+                            for _ in 0..action_count {
+                                ds.message(t, &(), &current_rec);
+                            }
+                            Ok(())
+                        });
                     }
                     Ok(LinkedTaskTermination::KeepFacet)
                 });
